function [ytrain,ytest] = KRR(X,y,lambda,sigma,Xtest)
% Train and predict Kernel Ridge Regression
% 
% Syntax:       [ytrain,ytest] = KRR(X,y,lambda,kernel,params,Xtest)
%
% Inputs:       X is a N x D matrix of training data
%               y is an N x 1 vector of labels
%               lambda is the ridge regression tuning parameter
%               sigma is the RBF kernel tuning parameter
%               Xtest is an optional Ntest x D matrix of test data
%
% Outputs:      ytrain is the set of predicted labels for the training data X
%               ytest is the set of predicted labels for the test data Xtest

[N,D] = size(X);
K = zeros(N);
for ii = 1:N
    for jj = 1:N
        %if strcmp(kernel,'rbf')
            K(ii,jj) = exp(-norm(X(ii,:)-X(jj,:))^2 / (2*sigma^2));
%        elseif strcmp(kernel,'poly')
            %K(ii,jj) = (X(ii,:)'*X(jj,:) + params(1))^params(2);
        %end
    end
end

w = (K + lambda*eye(N))\y;
ytrain = K*w;

if nargin == 5
    Ntest = size(Xtest,1);
    ytest = zeros(Ntest,1);
    for ii = 1:Ntest
        k = zeros(N,1);
        for jj = 1:N
            %if strcmp(kernel,'rbf')
                k(jj) = exp(-norm(Xtest(ii,:)-X(jj,:))^2 / (2*sigma^2));
            %elseif strcmp(kernel,'poly')
                %k(jj) = (Xtest(ii,:)'*X(jj,:) + params(1))^params(2);
            %end
            ytest(ii) = w'*k;
        end
    end
else
    ytest = [];
end
