"""
KRR demo - KRR.py

ECE 510

python version: 3.7.2

Spring 2019
"""

import numpy as np
from numpy import exp, eye
from numpy.linalg import norm, inv

def KRR(X, y, lambda_, sigma, Xtest=None):
    r"""
    Train and predict Kernel Ridge Regression

    Syntax: ytrain, ytest = KRR(X, y, lambda, kernel, params, Xtest)
    
    Inputs: 
        :param X: A N x D matrix of training data
        :param lambda: The ridge regression tuning parameter
        :param sigma: The RBF kernel tuning parameter
        :param Xtest: Optional matrix of test data
    
    Outputs:
        ytrain is the set of predicted labels for the training data X
        ytest is the set of predicted labels for the test data Xtest
    """

    ## Task 2
    N, _ = X.shape
    K = np.zeros([N, N])

    for ii in range(N):
        for jj in range(N):
            K[ii,jj] = exp(-1.0 * norm(X[ii, :] - X[jj, :])**2 / (2*sigma**2))
    
    w = inv(K + lambda_ * eye(N)) @ y
    ytrain = K @ w

    ## Task 3
    if not Xtest is None:
        Ntest = np.shape(Xtest)[0]
        ytest = np.zeros([Ntest, 1])
        for ii in range(Ntest):
            k = np.zeros([N, 1])
            for jj in range(N):
                k[jj] = exp(-1.0 * norm(Xtest[ii, :] - X[jj, :])**2 / (2*sigma**2))
                ytest[ii] = w.T @ k
    else:
        ytest=[]

    return ytrain, ytest
